/**
 * The sort descriptor used by the `orderBy` method.
 */
export interface SortDescriptor {
    /**
     * The field that is sorted.
     */
    field: string;
    /**
     * The sort direction. If no direction is set, the descriptor will be skipped during processing.
     *
     * The available values are:
     * - `asc`
     * - `desc`
     */
    dir?: 'asc' | 'desc';
    /**
     * A custom function that defines the sort order.
     *
     * @param a - The first element for comparison.
     * @param b - The second element for comparison.
     *
     * @returns The return value should be a number whose sign indicates the relative order of the two elements:
     * negative if `a` is less than `b`, positive if `a` is greater than `b`, and zero if they are equal. NaN is treated as 0.
     */
    compare?: (a: any, b: any) => number;
}
